@@ -34,12 +34,17 @@ namespace BerkeleyDB {
         /// If true, the site is client-to-client peer.
         /// </summary>
         public bool isPeer;
+        /// <summary>
+        /// If true, the site is a view site.
+        /// </summary>
+        public bool isView;
 
         internal RepMgrSite(DB_REPMGR_SITE site) {
             EId = site.eid;
             Address = new ReplicationHostAddress(site.host, site.port);
             isConnected = (site.status == DbConstants.DB_REPMGR_CONNECTED);
-            isPeer = (site.flags & DbConstants.DB_REPMGR_PEER) != 0;
+            isPeer = (site.flags & DbConstants.DB_REPMGR_ISPEER) != 0;
+            isView = (site.flags & DbConstants.DB_REPMGR_ISVIEW) != 0;
         }
     }
 }
