@@ -365,6 +365,14 @@ it will be closed when the environment handle that owns the database handle is c
             DbConstants.DB_TXN_NOSYNC : 0) : 0);
     }
 
+    private java.io.File getBlobSubDir()
+        throws DatabaseException {
+        String blobDirStr = db.get_blob_sub_dir();
+        if (blobDirStr != null)
+            return (new java.io.File(blobDirStr));
+        return null;
+    }
+
     /**
 Return the database's underlying file name.
 <p>
@@ -1360,7 +1368,8 @@ The database to be removed.
                               DatabaseConfig config)
         throws DatabaseException, java.io.FileNotFoundException {
 
-        final Db db = DatabaseConfig.checkNull(config).createDatabase(null);
+        final Db db = DatabaseConfig.DEFAULT.createDatabase(null);
+        DatabaseConfig.checkNull(config).configureDatabase(db, DatabaseConfig.DEFAULT);
         db.remove(fileName, databaseName, 0);
     }
 
@@ -1637,6 +1646,8 @@ or failure.
         throws DatabaseException, java.io.FileNotFoundException {
 
         final Db db = DatabaseConfig.checkNull(dbConfig).createDatabase(null);
+        /* Configure db with dbConfig */
+        dbConfig.configureDatabase(db, DatabaseConfig.DEFAULT);
         return db.verify(fileName, databaseName, dumpStream,
             VerifyConfig.checkNull(verifyConfig).getFlags());
     }
