@@ -472,7 +472,7 @@ static int backupCleanup(sqlite3_backup *p)
 		if (p->rc == SQLITE_DONE)
 			rc2 = sqlite3BtreeCommit(p->pDest);
 		else
-			rc2 = sqlite3BtreeRollback(p->pDest);
+			rc2 = sqlite3BtreeRollback(p->pDest, SQLITE_OK);
 		if (rc2 != SQLITE_OK)
 			rc = rc2;
 	}
@@ -487,7 +487,6 @@ static int backupCleanup(sqlite3_backup *p)
 		 */
 		sqlite3_snprintf(sizeof(path), path,
 		    "%s%s", p->fullName, BACKUP_SUFFIX);
-		p->pDest->schema = NULL;
 		if (p->rc == SQLITE_DONE) {
 			rc2 = btreeDeleteEnvironment(p->pDest, path, 0);
 		} else {
@@ -495,25 +494,26 @@ static int backupCleanup(sqlite3_backup *p)
 			if (!__os_exists(NULL, path, 0))
 				__os_rename(NULL, path, p->fullName, 0);
 		}
+		if (rc2 != SQLITE_BUSY) {
+		    p->pDest = p->pDestDb->aDb[p->iDb].pBt = NULL;
+		    p->pDestDb->aDb[p->iDb].pSchema = NULL;
+		}
 		if (rc == SQLITE_OK)
 			rc = rc2;
 		if (rc == SQLITE_OK) {
 			p->pDest = NULL;
 			p->pDestDb->aDb[p->iDb].pBt = NULL;
 			p->openDest = 0;
-			rc = sqlite3BtreeOpen(p->fullName, p->pDestDb,
+			rc = sqlite3BtreeOpen(NULL, p->fullName, p->pDestDb,
 			    &p->pDest,
 			    SQLITE_DEFAULT_CACHE_SIZE | SQLITE_OPEN_MAIN_DB,
 			    p->pDestDb->openFlags);
 			p->pDestDb->aDb[p->iDb].pBt = p->pDest;
-			if (p->pDest)
-				p->pDest->schema = 
-				    p->pDestDb->aDb[p->iDb].pSchema;
-			else {
-				sqlite3SchemaClear(
-				    p->pDestDb->aDb[p->iDb].pSchema);
-				sqlite3_free(p->pDestDb->aDb[p->iDb].pSchema);
-				p->pDestDb->aDb[p->iDb].pSchema = NULL;
+			if (p->pDest) {
+				p->pDestDb->aDb[p->iDb].pSchema = 
+				    sqlite3SchemaGet(p->pDestDb, p->pDest);
+				if (p->pDestDb->aDb[p->iDb].pSchema == NULL)
+					rc = SQLITE_NOMEM;
 			}
 			if (rc == SQLITE_OK)
 				p->pDest->pBt->db_oflags |= DB_CREATE;
@@ -607,15 +607,19 @@ int sqlite3_backup_step(sqlite3_backup *p, int nPage) {
 		storage = p->pDest->pBt->dbStorage;
 		if (storage == DB_STORE_NAMED)
 			p->openDest = 1;
-		if (p->pDest)
+		if (strcmp(p->destName, "temp") == 0)
 			p->pDest->schema = NULL;
+		else 
+			p->pDestDb->aDb[p->iDb].pSchema = NULL;
+			
 		p->rc = btreeDeleteEnvironment(p->pDest, p->fullName, 1);
 		if (storage == DB_STORE_INMEM && strcmp(p->destName, "temp")
 		    != 0)
 			home = inmem;
 		else
 			home = p->fullName;
-		p->pDest = p->pDestDb->aDb[p->iDb].pBt;
+		if (p->rc != SQLITE_BUSY)
+			p->pDest = p->pDestDb->aDb[p->iDb].pBt = NULL;
 		if (p->rc != SQLITE_OK)
 			goto err;
 		/*
@@ -629,19 +633,19 @@ int sqlite3_backup_step(sqlite3_backup *p, int nPage) {
 			parse.db = p->pDestDb;
 			p->rc = sqlite3OpenTempDatabase(&parse);
 			p->pDest = p->pDestDb->aDb[p->iDb].pBt;
+			if (p->pDest && p->iDb != 1)
+				p->pDest->schema =
+				    p->pDestDb->aDb[p->iDb].pSchema;
 		} else {
-			p->rc = sqlite3BtreeOpen(home, p->pDestDb,
+			p->rc = sqlite3BtreeOpen(NULL, home, p->pDestDb,
 			    &p->pDest, SQLITE_DEFAULT_CACHE_SIZE |
 			    SQLITE_OPEN_MAIN_DB, p->pDestDb->openFlags);
 			p->pDestDb->aDb[p->iDb].pBt = p->pDest;
-			if (p->pDest)
-				p->pDest->schema =
-				    p->pDestDb->aDb[p->iDb].pSchema;
-			else {
-				sqlite3SchemaClear(
-				    p->pDestDb->aDb[p->iDb].pSchema);
-				sqlite3_free(p->pDestDb->aDb[p->iDb].pSchema);
-				p->pDestDb->aDb[p->iDb].pSchema = NULL;
+			if (p->pDest) {
+				p->pDestDb->aDb[p->iDb].pSchema = 
+				    sqlite3SchemaGet(p->pDestDb, p->pDest);
+				if (p->pDestDb->aDb[p->iDb].pSchema == NULL)
+					p->rc = SQLITE_NOMEM;
 			}
 		}
 
@@ -725,7 +729,7 @@ int sqlite3_backup_step(sqlite3_backup *p, int nPage) {
 	p->rc = btreeCopyPages(p, &pages);
 	if (p->rc == SQLITE_DONE) {
 		p->nRemaining = 0;
-		sqlite3ResetInternalSchema(p->pDestDb, p->iDb);
+		sqlite3ResetOneSchema(p->pDestDb, p->iDb);
 		memset(&parse, 0, sizeof(parse));
 		parse.db = p->pDestDb;
 		p->rc = sqlite3ReadSchema(&parse);
