@@ -237,7 +237,6 @@ tcl_LockStat(interp, objc, objv, dbenv)
 	 * list pairs and free up the memory.
 	 */
 	res = Tcl_NewObj();
-#ifdef HAVE_STATISTICS
 	/*
 	 * MAKE_STAT_LIST assumes 'res' and 'error' label.
 	 */
@@ -266,6 +265,9 @@ tcl_LockStat(interp, objc, objv, dbenv)
 	    sp->st_maxlsteals);
 	MAKE_STAT_LIST("Current number of lockers", sp->st_nlockers);
 	MAKE_STAT_LIST("Maximum number of lockers so far", sp->st_maxnlockers);
+	MAKE_STAT_LIST("Number of hits in the thread locker cache",
+	    sp->st_nlockers_hit);
+	MAKE_STAT_LIST("Total number of lockers reused", sp->st_nlockers_reused);
 	MAKE_STAT_LIST("Current number of objects", sp->st_nobjects);
 	MAKE_STAT_LIST("Maximum number of objects so far", sp->st_maxnobjects);
 	MAKE_STAT_LIST("Maximum number of objects in any hash bucket",
@@ -304,7 +306,7 @@ tcl_LockStat(interp, objc, objv, dbenv)
 	    sp->st_part_max_wait);
 	MAKE_STAT_LIST("Maximum number nowaits on any lock partition mutex",
 	    sp->st_part_max_nowait);
-#endif
+
 	Tcl_SetObjResult(interp, res);
 error:
 	__os_ufree(dbenv->env, sp);
