@@ -43,6 +43,9 @@ __db_vrfy_dbinfo_create(env, ip, pgsize, vdpp)
 	if ((ret = __db_create_internal(&cdbp, env, 0)) != 0)
 		goto err;
 
+	if ((ret = __db_set_blob_threshold(cdbp, 0, 0)) != 0)
+		goto err;
+
 	if ((ret = __db_set_flags(cdbp, DB_DUP)) != 0)
 		goto err;
 
@@ -60,6 +63,9 @@ __db_vrfy_dbinfo_create(env, ip, pgsize, vdpp)
 	if ((ret = __db_create_internal(&pgdbp, env, 0)) != 0)
 		goto err;
 
+	if ((ret = __db_set_blob_threshold(pgdbp, 0, 0)) != 0)
+		goto err;
+
 	if ((ret = __db_set_pagesize(pgdbp, pgsize)) != 0)
 		goto err;
 
@@ -928,5 +934,6 @@ __db_vrfy_prdbt(dbtp, checkprint, prefix,
 	}
 	return (
 	    __db_prdbt(dbtp, checkprint,
-	    prefix, handle, callback, is_recno, is_heap));
+	    prefix, handle, callback, is_recno, is_heap,
+	    F_ISSET(vdp, SALVAGE_STREAM_BLOB) ? 1 : 0));
 }
