@@ -14,6 +14,10 @@
  * __os_gettime --
  *	Return the current time-of-day clock in seconds and nanoseconds.
  *
+ * If you want the time of day, pass 0 in the monotonic argument.  If you pass
+ * non-zero, you might get time-of-day or you might get a non-decreasing number
+ * which is unrelated to the time of day, such as the seconds since system boot.
+ *
  * PUBLIC: void __os_gettime __P((ENV *, db_timespec *, int));
  */
 void
@@ -35,7 +39,6 @@ __os_gettime(env, tp, monotonic)
 		RETRY_CHK((clock_gettime(
 		    CLOCK_REALTIME, (struct timespec *)tp)), ret);
 
-	RETRY_CHK((clock_gettime(CLOCK_REALTIME, (struct timespec *)tp)), ret);
 	if (ret != 0) {
 		sc = "clock_gettime";
 		goto err;
