@@ -466,8 +466,8 @@ __rep_log_split(env, ip, rp, rec, ret_lsnp, last_lsnp)
 		if (p >= ep && save_flags)
 			F_SET(&tmprp, save_flags);
 		/*
-		 * A previous call to __rep_apply indicated an earlier
-		 * record is a dup and the next_new_lsn we are waiting for.
+		 * A previous call to __rep_apply indicated an earlier record
+		 * is a past dup and the next_new_lsn for which we are waiting.
 		 * Skip log records until we catch up with next_new_lsn.
 		 */
 		if (is_dup && LOG_COMPARE(&tmprp.lsn, &next_new_lsn) < 0) {
@@ -482,7 +482,20 @@ __rep_log_split(env, ip, rp, rec, ret_lsnp, last_lsnp)
 		VPRINT(env, (env, DB_VERB_REP_MISC,
 		    "log_split: rep_apply ret %d, dup %d, tmp_lsn [%lu][%lu]",
 		    ret, is_dup, (u_long)tmp_lsn.file, (u_long)tmp_lsn.offset));
-		if (is_dup)
+		/*
+		 * We can skip log records between a past dup and tmp_lsn
+		 * returned by rep_apply() because we know we have all
+		 * those log records.  For a past dup, this log record is
+		 * less than or equal to tmp_lsn (which is either ready_lsn
+		 * or max_perm_lsn) and we only have records to skip when
+		 * it is less than tmp_lsn.
+		 *
+		 * We cannot skip log records for a future dup because we
+		 * may not have all of them.  In this case, this log record
+		 * is greater than or equal to tmp_lsn (which is either
+		 * ready_lsn or this log record).
+		 */
+		if (is_dup && LOG_COMPARE(&tmprp.lsn, &tmp_lsn) < 0)
 			next_new_lsn = tmp_lsn;
 		switch (ret) {
 		/*
@@ -811,6 +824,14 @@ __rep_loggap_req(env, rep, lsnp, gapflags)
 	type = REP_LOG_REQ;
 	ret = 0;
 
+	/*
+	 * If we are in SYNC_LOG and have all the log we need (i.e.
+	 * rep->last_lsn is ZERO_LSN), just return, as there is nothing
+	 * to do while recovery is running.
+	 */
+	if (rep->sync_state == SYNC_LOG && IS_ZERO_LSN(rep->last_lsn))
+		return (0);
+
 	/*
 	 * Check if we need to ask for the gap.
 	 * We ask for the gap if:
