@@ -73,6 +73,13 @@ static const char *__rep_syncstate_to_string __P((repsync_t));
 	}								\
 } while (0)
 
+#define	PRINT_VIEW(sp) do {						\
+	if ((sp)->st_view != 0)						\
+		__db_msg(env, "Environment configured as view site");	\
+	else								\
+		__db_msg(env, "Environment not configured as view site");\
+} while (0)
+
 /*
  * __rep_stat_pp --
  *	ENV->rep_stat pre/post processing.
@@ -120,7 +127,7 @@ __rep_stat(env, statp, flags)
 	DB_REP_STAT *stats;
 	LOG *lp;
 	REP *rep;
-	u_int32_t startupdone;
+	u_int32_t startupdone, view;
 	uintmax_t queued;
 	int dolock, ret;
 
@@ -177,10 +184,12 @@ __rep_stat(env, statp, flags)
 	if (LF_ISSET(DB_STAT_CLEAR)) {
 		queued = rep->stat.st_log_queued;
 		startupdone = rep->stat.st_startup_complete;
+		view = rep->stat.st_view;
 		memset(&rep->stat, 0, sizeof(rep->stat));
 		rep->stat.st_log_queued = rep->stat.st_log_queued_total =
 		    rep->stat.st_log_queued_max = queued;
 		rep->stat.st_startup_complete = startupdone;
+		rep->stat.st_view = view;
 	}
 
 	/*
@@ -377,6 +386,7 @@ __rep_print_stats(env, flags)
 	__db_dl(env, "Number of page records missed and requested",
 	    (u_long)sp->st_pg_requested);
 	PRINT_STARTUPCOMPLETE(sp);
+	PRINT_VIEW(sp);
 	__db_dl(env,
 	    "Number of transactions applied", (u_long)sp->st_txns_applied);
 
@@ -465,6 +475,7 @@ __rep_print_all(env, flags)
 		{ REP_C_2SITE_STRICT,	"REP_C_2SITE_STRICT" },
 		{ REP_C_AUTOINIT,	"REP_C_AUTOINIT" },
 		{ REP_C_AUTOROLLBACK,	"REP_C_AUTOROLLBACK" },
+		{ REP_C_AUTOTAKEOVER,	"REP_C_AUTOTAKEOVER" },
 		{ REP_C_BULK,		"REP_C_BULK" },
 		{ REP_C_DELAYCLIENT,	"REP_C_DELAYCLIENT" },
 		{ REP_C_ELECTIONS,	"REP_C_ELECTIONS" },
@@ -523,15 +534,16 @@ __rep_print_all(env, flags)
 	rep = db_rep->region;
 	infop = env->reginfo;
 	renv = infop->primary;
-	ENV_ENTER(env, ip);
 
 	__db_msg(env, "%s", DB_GLOBAL(db_line));
 	__db_msg(env, "DB_REP handle information:");
 
 	if (db_rep->rep_db == NULL)
 		STAT_ISSET("Bookkeeping database", db_rep->rep_db);
-	else
+	else {
+		ENV_GET_THREAD_INFO(env, ip);
 		(void)__db_stat_print(db_rep->rep_db, ip, flags);
+	}
 
 	__db_prflags(env, NULL, db_rep->flags, dbrep_fn, NULL, "\tFlags");
 
@@ -604,7 +616,6 @@ __rep_print_all(env, flags)
 	STAT_LONG("Maximum lease timestamp microseconds",
 	    lp->max_lease_ts.tv_nsec / NS_PER_US);
 	MUTEX_UNLOCK(env, rep->mtx_clientdb);
-	ENV_LEAVE(env, ip);
 
 	return (0);
 }
@@ -648,8 +659,10 @@ __rep_stat_summary_print(env)
 	ret = 0;
 	if ((ret = __rep_stat(env, &sp, 0)) == 0) {
 		PRINT_STATUS(sp, is_client);
-		if (is_client)
+		if (is_client) {
 			PRINT_STARTUPCOMPLETE(sp);
+			PRINT_VIEW(sp);
+		}
 		PRINT_MAXPERMLSN(sp);
 		/*
 		 * Use the number of sites that is kept up-to-date most
