@@ -7,59 +7,39 @@
 
 package com.sleepycat.bind.tuple.test;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.Arrays;
 
-import junit.framework.Test;
-import junit.framework.TestCase;
-import junit.framework.TestSuite;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
 
 import com.sleepycat.bind.tuple.TupleBinding;
 import com.sleepycat.bind.tuple.TupleInput;
 import com.sleepycat.bind.tuple.TupleOutput;
 import com.sleepycat.db.DatabaseEntry;
-import com.sleepycat.util.test.SharedTestUtils;
 
 /**
  * @author Mark Hayes
  */
-public class TupleFormatTest extends TestCase {
+public class TupleFormatTest {
 
     private TupleInput in;
     private TupleOutput out;
     private DatabaseEntry buffer;
 
-    public static void main(String[] args) {
-        junit.framework.TestResult tr =
-            junit.textui.TestRunner.run(suite());
-        if (tr.errorCount() > 0 ||
-            tr.failureCount() > 0) {
-            System.exit(1);
-        } else {
-            System.exit(0);
-        }
-    }
-
-    public static Test suite() {
-        TestSuite suite = new TestSuite(TupleFormatTest.class);
-        return suite;
-    }
-
-    public TupleFormatTest(String name) {
-
-        super(name);
-    }
-
-    @Override
+    @Before
     public void setUp() {
 
-        SharedTestUtils.printTestName("TupleFormatTest." + getName());
         buffer = new DatabaseEntry();
         out = new TupleOutput();
     }
 
-    @Override
+    @After
     public void tearDown() {
 
         /* Ensure that GC can cleanup. */
@@ -87,6 +67,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(0, in.available());
     }
 
+    @Test
     public void testString() {
 
         stringTest("");
@@ -130,6 +111,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(0, in.available());
     }
 
+    @Test
     public void testFixedString() {
 
         fixedStringTest(new char[0]);
@@ -157,6 +139,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(0, in.available());
     }
 
+    @Test
     public void testNullString() {
 
         out.reset();
@@ -229,6 +212,7 @@ public class TupleFormatTest extends TestCase {
         }
     }
 
+    @Test
     public void testChars() {
 
         charsTest(new char[0]);
@@ -282,6 +266,7 @@ public class TupleFormatTest extends TestCase {
         }
     }
 
+    @Test
     public void testBytes() {
 
         bytesTest(new char[0]);
@@ -320,6 +305,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(0, in.available());
     }
 
+    @Test
     public void testBoolean() {
 
         booleanTest(true);
@@ -360,6 +346,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(expected, in.readUnsignedByte());
     }
 
+    @Test
     public void testUnsignedByte() {
 
         unsignedByteTest(0);
@@ -397,6 +384,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(expected, in.readUnsignedShort());
     }
 
+    @Test
     public void testUnsignedShort() {
 
         unsignedShortTest(0);
@@ -442,6 +430,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(expected, in.readUnsignedInt());
     }
 
+    @Test
     public void testUnsignedInt() {
 
         unsignedIntTest(0L);
@@ -485,6 +474,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals((byte) val, in.readByte());
     }
 
+    @Test
     public void testByte() {
 
         byteTest(0);
@@ -524,6 +514,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals((short) val, in.readShort());
     }
 
+    @Test
     public void testShort() {
 
         shortTest(0);
@@ -563,6 +554,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readInt());
     }
 
+    @Test
     public void testInt() {
 
         intTest(0);
@@ -602,6 +594,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readLong());
     }
 
+    @Test
     public void testLong() {
 
         longTest(0);
@@ -647,6 +640,7 @@ public class TupleFormatTest extends TestCase {
         }
     }
 
+    @Test
     public void testFloat() {
 
         floatTest(0);
@@ -704,6 +698,7 @@ public class TupleFormatTest extends TestCase {
         }
     }
 
+    @Test
     public void testDouble() {
 
         doubleTest(0);
@@ -764,6 +759,7 @@ public class TupleFormatTest extends TestCase {
         }
     }
 
+    @Test
     public void testSortedFloat() {
 
         sortedFloatTest(0);
@@ -821,6 +817,7 @@ public class TupleFormatTest extends TestCase {
         }
     }
 
+    @Test
     public void testSortedDouble() {
 
         sortedDoubleTest(0);
@@ -878,6 +875,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readPackedInt());
     }
 
+    @Test
     public void testPackedInt() {
 
         /* Exhaustive value testing is in PackedIntTest. */
@@ -907,6 +905,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readPackedLong());
     }
 
+    @Test
     public void testPackedLong() {
 
         /* Exhaustive value testing is in PackedIntTest. */
@@ -936,6 +935,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readSortedPackedInt());
     }
     
+    @Test
     public void testSortedPackedInt() {
 
         /* Exhaustive value testing is in sortedPackedIntTest. */
@@ -987,6 +987,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readSortedPackedLong());
     }
     
+    @Test
     public void testSortedPackedLong() {
 
         /* Exhaustive value testing is in sortedPackedLongTest. */
@@ -1053,6 +1054,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readBigInteger());
     }
     
+    @Test
     public void testBigInteger() {
 
         /* Exhaustive value testing is in bigIntegerTest. */
@@ -1094,6 +1096,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(val, in.readBigDecimal());
     }
     
+    @Test
     public void testBigDecimal() {
 
         /* Exhaustive value testing is in BigDecimal. */
@@ -1145,6 +1148,7 @@ public class TupleFormatTest extends TestCase {
         assertEquals(0, val.compareTo(in.readSortedBigDecimal()));
     }
     
+    @Test
     public void testSortedBigDecimal() {
 
         /* Exhaustive value testing is in BigDecimal. */
