@@ -13,6 +13,12 @@ import static com.sleepycat.persist.model.Relationship.MANY_TO_MANY;
 import static com.sleepycat.persist.model.Relationship.MANY_TO_ONE;
 import static com.sleepycat.persist.model.Relationship.ONE_TO_MANY;
 import static com.sleepycat.persist.model.Relationship.ONE_TO_ONE;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.util.ArrayList;
 import java.util.EnumSet;
@@ -22,7 +28,11 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import junit.framework.Test;
+import org.junit.After;
+import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.Parameterized;
+import org.junit.runners.Parameterized.Parameters;
 
 import com.sleepycat.compat.DbCompat;
 import com.sleepycat.db.Database;
@@ -52,13 +62,22 @@ import com.sleepycat.util.test.TxnTestCase;
  *
  * @author Mark Hayes
  */
+@RunWith(Parameterized.class)
 public class OperationTest extends TxnTestCase {
 
     private static final String STORE_NAME = "test";
 
-    public static Test suite() {
-        testClass = OperationTest.class;
-        return txnTestSuite(null, null);
+    @Parameters
+    public static List<Object[]> genParams() {
+        
+        return getTxnTypes(null, false);
+    }
+    
+    public OperationTest(String type){
+        initEnvConfig();
+        txnType = type;
+        isTransactional = (txnType != TXN_NULL);
+        customName = txnType;
     }
 
     private EntityStore store;
@@ -105,17 +124,10 @@ public class OperationTest extends TxnTestCase {
         store = null;
     }
 
-    @Override
-    public void setUp()
-        throws Exception {
-
-        super.setUp();
-    }
-
     /**
      * The store must be closed before closing the environment.
      */
-    @Override
+    @After
     public void tearDown()
         throws Exception {
 
@@ -130,6 +142,7 @@ public class OperationTest extends TxnTestCase {
         super.tearDown();
     }
 
+    @Test
     public void testReadOnly()
         throws DatabaseException {
 
@@ -157,6 +170,7 @@ public class OperationTest extends TxnTestCase {
 
 
 
+    @Test
     public void testUninitializedCursor()
         throws DatabaseException {
 
@@ -204,6 +218,7 @@ public class OperationTest extends TxnTestCase {
         close();
     }
 
+    @Test
     public void testCursorCount()
         throws DatabaseException {
 
@@ -238,6 +253,7 @@ public class OperationTest extends TxnTestCase {
         close();
     }
 
+    @Test
     public void testCursorUpdate()
         throws DatabaseException {
 
@@ -321,6 +337,7 @@ public class OperationTest extends TxnTestCase {
         close();
     }
 
+    @Test
     public void testCursorDelete()
         throws DatabaseException {
 
@@ -426,6 +443,7 @@ public class OperationTest extends TxnTestCase {
         close();
     }
 
+    @Test
     public void testDeleteFromSubIndex()
         throws DatabaseException {
 
@@ -503,6 +521,7 @@ public class OperationTest extends TxnTestCase {
         private MyEntity() {}
     }
 
+    @Test
     public void testSharedSequence()
         throws DatabaseException {
 
@@ -546,6 +565,7 @@ public class OperationTest extends TxnTestCase {
         private Integer key;
     }
 
+    @Test
     public void testSeparateSequence()
         throws DatabaseException {
 
@@ -591,6 +611,7 @@ public class OperationTest extends TxnTestCase {
         private Integer key;
     }
 
+    @Test
     public void testCompositeSequence()
         throws DatabaseException {
 
@@ -699,6 +720,7 @@ public class OperationTest extends TxnTestCase {
      * NullPointerException in JE 3.0.12.  No SR was created because the use
      * case is very obscure and was discovered by code inspection.
      */
+    @Test
     public void testOpenRawStoreReadOnly()
         throws DatabaseException {
 
@@ -723,6 +745,7 @@ public class OperationTest extends TxnTestCase {
      * referenced when getSecondaryIndex was called.  This was a bug in JE
      * 3.0.12, reported on OTN.  [#15103]
      */
+    @Test
     public void testToManyKeyClass()
         throws DatabaseException {
 
@@ -744,6 +767,7 @@ public class OperationTest extends TxnTestCase {
      * fail with "IllegalArgumentException: Wrong secondary key class: ..."
      * when the store was opened read-only.  [#15156]
      */
+    @Test
     public void testToManyReadOnly()
         throws DatabaseException {
 
@@ -790,6 +814,7 @@ public class OperationTest extends TxnTestCase {
      * be opened automatically.  If X is not opened, foreign key constraints
      * will not be enforced. [#15358]
      */
+    @Test
     public void testAutoOpenRelatedEntity()
         throws DatabaseException {
 
@@ -854,12 +879,14 @@ public class OperationTest extends TxnTestCase {
         }
     }
 
+    @Test
     public void testSecondaryBulkLoad1()
         throws DatabaseException {
 
         doSecondaryBulkLoad(true);
     }
 
+    @Test
     public void testSecondaryBulkLoad2()
         throws DatabaseException {
 
@@ -945,6 +972,7 @@ public class OperationTest extends TxnTestCase {
         close();
     }
 
+    @Test
     public void testPersistentFields()
         throws DatabaseException {
 
@@ -1004,6 +1032,7 @@ public class OperationTest extends TxnTestCase {
      * getSecondaryConfig.  This was a bug in JE 3.3.69, reported on OTN.
      * [#16407]
      */
+    @Test
     public void testKeyClassInitialization()
         throws DatabaseException {
 
@@ -1012,6 +1041,7 @@ public class OperationTest extends TxnTestCase {
         close();
     }
 
+    @Test
     public void testKeyName()
         throws DatabaseException {
 
@@ -1063,6 +1093,7 @@ public class OperationTest extends TxnTestCase {
      * subclass instance, which contains a secondary key, without registering
      * the subclass up front. [#16399]
      */
+    @Test
     public void testPutEntitySubclassWithoutRegisterClass()
         throws DatabaseException {
 
@@ -1092,6 +1123,7 @@ public class OperationTest extends TxnTestCase {
      * Checks that registerClass avoids an exception when storing an entity
      * subclass instance, which defines a secondary key. [#16399]
      */
+    @Test
     public void testPutEntitySubclassWithRegisterClass()
         throws DatabaseException {
 
@@ -1123,6 +1155,7 @@ public class OperationTest extends TxnTestCase {
      * registerClass is sufficient and subsequent use of the store does not
      * require it.  [#16399]
      */
+    @Test
     public void testPutEntitySubclassWithRegisterClass2()
         throws DatabaseException {
 
@@ -1160,6 +1193,7 @@ public class OperationTest extends TxnTestCase {
      * avoid an exception when storing an entity subclass instance, which
      * defines a secondary key. [#16399]
      */
+    @Test
     public void testPutEntitySubclassWithGetSubclassIndex()
         throws DatabaseException {
 
@@ -1191,6 +1225,7 @@ public class OperationTest extends TxnTestCase {
      * getSubclassIndex is sufficient and subsequent use of the store does not
      * require it.  [#16399]
      */
+    @Test
     public void testPutEntitySubclassWithGetSubclassIndex2()
         throws DatabaseException {
 
@@ -1260,6 +1295,7 @@ public class OperationTest extends TxnTestCase {
         private ExtendedStatement() {}
     } 
 
+    @Test
     public void testCustomCompare()
         throws DatabaseException {
 
@@ -1329,7 +1365,7 @@ public class OperationTest extends TxnTestCase {
         private ReverseIntKey secKey1;
 
         @SecondaryKey(relate=ONE_TO_MANY)
-        private Set<ReverseIntKey> secKey2 = new HashSet<ReverseIntKey>();
+        private final Set<ReverseIntKey> secKey2 = new HashSet<ReverseIntKey>();
 
         private CustomCompareEntity() {}
 
@@ -1390,6 +1426,7 @@ public class OperationTest extends TxnTestCase {
      * comparator.  The comparator is initialized on its first use, just as if
      * recovery were run.
      */
+    @Test
     public void testStoredComparators()
         throws DatabaseException {
 
@@ -1562,6 +1599,7 @@ public class OperationTest extends TxnTestCase {
         }
     }
     
+    @Test
     public void testEmbeddedMapTypes()
         throws DatabaseException {
         open();
