@@ -1216,6 +1216,7 @@ proc run_election { celist errcmd priority crsh\
 
 	global elect_serial
 	global is_hp_test
+	global is_sunos_test
 	global is_windows_test
 	global rand_init
 	upvar $celist cenvlist
@@ -1223,9 +1224,9 @@ proc run_election { celist errcmd priority crsh\
 	upvar $priority pri
 	upvar $crsh crash
 
-	# Windows and HP-UX require a longer timeout.
-	if { [llength $elect_timeout] == 1 &&
-	    ($is_windows_test == 1 || $is_hp_test == 1) } {
+	# Windows, HP-UX and SunOS require a longer timeout.
+	if { [llength $elect_timeout] == 1 && ($is_windows_test == 1 ||
+	    $is_hp_test == 1 || $is_sunos_test == 1) } {
 		set elect_timeout [expr $elect_timeout * 2]
 	}
 
@@ -2911,3 +2912,301 @@ proc rep_client_access { env testfile result } {
 		error_check_good clacc_close [$res close] 0
 	}
 }
+
+#
+# View function for replication.
+# This function always returns 0 and does not replicate any database files.
+#
+proc replview_none { name flags } {
+	# Verify flags are always 0 - "none" in Tcl.
+#	puts "Replview_none called with $name, $flags"
+	set noflags [string compare $flags "none"]
+	error_check_good chkflags $noflags 0
+
+	# Verify we never get a BDB owned file.
+	set bdbfile "__db"
+	set prefix_len [string length $bdbfile]
+	incr prefix_len -1
+	set substr [string range $name 0 $prefix_len]
+	set res [string compare $substr $bdbfile]
+	error_check_bad notbdbfile $res 0
+
+	#
+	# Otherwise this proc always returns 0 to say we do not want the file.
+	#
+	return 0
+}
+
+#
+# View function for replication.
+# This function returns 1 if the name has an odd digit in it, and 0
+# otherwise.
+#
+proc replview_odd { name flags } {
+#	puts "Replview_odd called with $name, $flags"
+
+	# Verify we never get a BDB owned file.
+	set bdbfile "__db"
+	set prefix_len [string length $bdbfile]
+	incr prefix_len -1
+	set substr [string range $name 0 $prefix_len]
+	set res [string compare $substr $bdbfile]
+	error_check_bad notbdbfile $res 0
+
+	#
+	# Otherwise look for an odd digit.
+	#
+	set odd [string match "*\[13579\]*" $name]
+	return $odd
+}
+
+#
+# Determine whether current version of Berkeley DB has group membership.
+# This function returns 1 if group membership is supported, and 0
+# otherwise.
+#
+proc have_group_membership { } {
+	set bdbver [berkdb version]
+	set vermaj [lindex $bdbver 0]
+	set vermin [lindex $bdbver 1]
+	if { $vermaj >= 6 } {
+		return 1
+	} elseif { $vermaj >= 5 && $vermin >= 2 } {
+		return 1
+	} else {
+		return 0
+	}
+}
+
+#
+# Create an empty marker file.  The upgrade tests use marker files to
+# synchronize between their different processes.
+#
+proc upgrade_create_markerfile { filename } {
+	if [catch {open $filename { RDWR CREAT } 0777} markid] {
+		puts "problem opening marker file $markid"
+	} else {
+		close $markid
+	}
+}
+
+proc upgrade_setup_sites { nsites } {
+	#
+	# Set up a list that goes from 0 to $nsites running
+	# upgraded.  A 0 represents running old version and 1
+	# represents running upgraded.  So, for 3 sites it will look like:
+	# { 0 0 0 } { 1 0 0 } { 1 1 0 } { 1 1 1 }
+	#
+	set sitelist {}
+	for { set i 0 } { $i <= $nsites } { incr i } {
+		set l ""
+		for { set j 1 } { $j <= $nsites } { incr j } {
+			if { $i < $j } {
+				lappend l 0
+			} else {
+				lappend l 1
+			}
+		}
+		lappend sitelist $l
+	}
+	return $sitelist
+}
+
+proc upgrade_one_site { histdir upgdir } {
+	global util_path
+
+	#
+	# Upgrade a site to the current version.  This entails:
+	# 1.  Removing any old files from the upgrade directory.
+	# 2.  Copy all old version files to upgrade directory.
+	# 3.  Remove any __db files from upgrade directory except __db.rep*gen.
+	# 4.  Force checkpoint in new version.
+	file delete -force $upgdir
+
+	# Recovery was run before as part of upgradescript.
+	# Archive dir by copying it to upgrade dir.
+	file copy -force $histdir $upgdir
+	set dbfiles [glob -nocomplain $upgdir/__db*]
+	foreach d $dbfiles {
+		if { $d == "$upgdir/__db.rep.gen" ||
+		    $d == "$upgdir/__db.rep.egen" ||
+		    $d == "$upgdir/__db.rep.system" } {
+			continue
+		}
+		file delete -force $d
+	}
+	# Force current version checkpoint
+	set stat [catch {eval exec $util_path/db_checkpoint -1 -h $upgdir} r]
+	if { $stat != 0 } {
+		puts "CHECKPOINT: $upgdir: $r"
+	}
+	error_check_good stat_ckp $stat 0
+}
+
+proc upgrade_get_master { nsites verslist } {
+	error_check_good vlist_chk [llength $verslist] $nsites
+	#
+	# When we can, simply run an election to get a new master.
+	# We then verify we got an old client.
+	#
+	# For now, randomly pick among the old sites, or if no old
+	# sites just randomly pick anyone.
+	#
+	set old_count 0
+	# Pick 1 out of N old sites or 1 out of nsites if all upgraded.
+	foreach i $verslist {
+		if { $i == 0 } {
+			incr old_count
+		}
+	}
+	if { $old_count == 0 } {
+		set old_count $nsites
+	}
+	set master [berkdb random_int 0 [expr $old_count - 1]]
+	#
+	# Since the Nth old site may not be at the Nth place in the
+	# list unless we used the entire list, we need to loop to find
+	# the right index to return.
+	if { $old_count == $nsites } {
+		return $master
+	}
+	set ocount 0
+	set index 0
+	foreach i $verslist {
+		if { $i == 1 } {
+			incr index
+			continue
+		}
+		if { $ocount == $master } {
+			return $index
+		}
+		incr ocount
+		incr index
+	}
+	#
+	# If we get here there is a problem in the code.
+	#
+	error "FAIL: upgrade_get_master problem"
+}
+
+# Shared upgrade test script procedure to execute rep_test_upg on a master.
+proc upgradescr_reptest { repenv oplist markerdir } {
+
+	set method [lindex $oplist 1]
+	set niter [lindex $oplist 2]
+	set loop [lindex $oplist 3]
+	set start 0
+	puts "REPTEST: method $method, niter $niter, loop $loop"
+
+	for {set n 0} {$n < $loop} {incr n} {
+		puts "REPTEST: call rep_test_upg $n"
+		eval rep_test_upg $method $repenv NULL $niter $start $start 0 0
+		incr start $niter
+		tclsleep 3
+	}
+	#
+	# Sleep a bunch to help get the messages worked through.
+	#
+	tclsleep 10
+	puts "create DONE marker file"
+	upgrade_create_markerfile $markerdir/DONE
+}
+
+# Shared upgrade test script procedure to perform db_gets on a client.
+proc upgradescr_repget { repenv oplist mydir markerdir } {
+	set dbname "$mydir/DATADIR/test.db"
+	set i 0
+	while { [file exists $dbname] == 0 } {
+		tclsleep 2
+		incr i
+		if { $i >= 15 && $i % 5 == 0 } {
+			puts "After $i seconds, no database $dbname exists."
+		}
+		if { $i > 180 } {
+			error "Database $dbname never created."
+		}
+	}
+	set loop 1
+	while { [file exists $markerdir/DONE] == 0 } {
+		set db [berkdb_open -env $repenv $dbname]
+		error_check_good dbopen [is_valid_db $db] TRUE
+		set dbc [$db cursor]
+		set i 0
+		error_check_good curs [is_valid_cursor $dbc $db] TRUE
+		for { set dbt [$dbc get -first ] } \
+		    { [llength $dbt] > 0 } \
+		    { set dbt [$dbc get -next] } {
+			incr i
+		}
+		error_check_good dbc_close [$dbc close] 0
+		error_check_good db_close [$db close] 0
+		puts "REPTEST_GET: after $loop loops: key count $i"
+		incr loop
+		tclsleep 2
+	}
+}
+
+# Shared upgrade test script procedure to verify dbs and logs.
+proc upgradescr_verify { oplist mydir rep_env_cmd } {
+	global util_path
+
+	# Change directories to where this will run.
+	# !!!
+	# mydir is an absolute path of the form
+	# <path>/build_unix/TESTDIR/MASTERDIR or
+	# <path>/build_unix/TESTDIR/CLIENTDIR.0
+	#
+	# So we want to run relative to the build_unix directory
+	cd $mydir/../..
+
+	foreach op $oplist {
+		set repenv [eval $rep_env_cmd]
+		error_check_good env_open [is_valid_env $repenv] TRUE
+		if { $op == "DB" } {
+			set dbname "$mydir/DATADIR/test.db"
+			puts "Open db: $dbname"
+			set db [berkdb_open -env $repenv -rdonly $dbname]
+			error_check_good dbopen [is_valid_db $db] TRUE
+			set txn ""
+			set method [$db get_type]
+			set dumpfile "$mydir/VERIFY/dbdump"
+			if { [is_record_based $method] == 1 } {
+				dump_file $db $txn $dumpfile \
+				    rep_test_upg.recno.check
+			} else {
+				dump_file $db $txn $dumpfile \
+				    rep_test_upg.check
+			}
+			puts "Done dumping $dbname to $dumpfile"
+			error_check_good dbclose [$db close] 0
+		}
+		if { $op == "LOG" } {
+			set lgstat [$repenv log_stat]
+			set lgfile [stat_field $repenv log_stat "Current log file number"]
+			set lgoff [stat_field $repenv log_stat "Current log file offset"]
+			puts "Current LSN: $lgfile $lgoff"
+			set f [open $mydir/VERIFY/loglsn w]
+			puts $f $lgfile
+			puts $f $lgoff
+			close $f
+
+			set stat [catch {eval exec $util_path/db_printlog \
+			    -h $mydir > $mydir/VERIFY/prlog} result]
+			if { $stat != 0 } {
+				puts "PRINTLOG: $result"
+			}
+			error_check_good stat_prlog $stat 0
+		}
+		error_check_good envclose [$repenv close] 0
+	}
+	#
+	# Run recovery locally so that any later upgrades are ready
+	# to be upgraded.
+	#
+	set stat [catch {eval exec $util_path/db_recover -h $mydir} result]
+	if { $stat != 0 } {
+		puts "RECOVERY: $result"
+	}
+	error_check_good stat_rec $stat 0
+
+}
