@@ -30,6 +30,7 @@ int env_init_print_42 __P((ENV *, DB_DISTAB *));
 int env_init_print_43 __P((ENV *, DB_DISTAB *));
 int env_init_print_47 __P((ENV *, DB_DISTAB *));
 int env_init_print_48 __P((ENV *, DB_DISTAB *));
+int env_init_print_53 __P((ENV *, DB_DISTAB *));
 int lsn_arg __P((char *, DB_LSN *));
 int main __P((int, char *[]));
 int open_rep_db __P((DB_ENV *, DB **, DBC **));
@@ -348,6 +349,8 @@ err:		exitval = 1;
 
 /*
  * env_init_print --
+ *
+ *	Fill the dispatch table for printing this log version's records.
  */
 int
 env_init_print(env, version, dtabp)
@@ -391,11 +394,16 @@ env_init_print(env, version, dtabp)
 	if ((ret = __txn_init_print(env, dtabp)) != 0)
 		goto err;
 
+	if (version == DB_LOGVERSION)
+		goto done;
+	/* DB_LOGVERSION_53 changed the heap addrem log record. */
+	if ((ret = env_init_print_53(env, dtabp)) != 0)
+		goto err;
 	/*
-	 * There are no log differences between 5.0 and 5.2, but 5.2
-	 * is a superset of 5.0.  Patch 2 of 4.8 added __db_pg_trunc
-	 * but didn't alter any log records so we want the same
-	 * override as 4.8
+	 * Since DB_LOGVERSION_53 is a strict superset of DB_LOGVERSION_50,
+	 * there is no need to check for log version between them; so only
+	 * check > DB_LOGVERSION_48p2.  Patch 2 of 4.8 added __db_pg_trunc but
+	 * didn't alter any log records so we want the same override as 4.8.
 	 */
 	if (version > DB_LOGVERSION_48p2)
 		goto done;
@@ -556,6 +564,23 @@ err:
 	return (ret);
 }
 
+int
+env_init_print_53(env, dtabp)
+	ENV *env;
+	DB_DISTAB *dtabp;
+{
+	int ret;
+#ifdef HAVE_HEAP
+	ret = __db_add_recovery_int(env,
+	     dtabp,__heap_addrem_50_print, DB___heap_addrem_50);
+#else
+	COMPQUIET(env, NULL);
+	COMPQUIET(dtabp, NULL);
+	COMPQUIET(ret, 0);
+#endif
+	return (ret);
+}
+
 int
 usage()
 {
